﻿using System;
[assembly: CLSCompliant(false)]
namespace Microsoft.ExternalResearch.CreativeCommons.Excel
{

    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Microsoft.Office.Tools.Ribbon;
    using System.Drawing;
    using Microsoft.ExternalResearch.CreativeCommons.Common;
    using System.Windows.Forms;
    using System.IO;
    using System.Runtime.InteropServices;
    using Microsoft.Office.Interop.Excel;
    using Microsoft.Office.Core;
    using System.Xml;
    using System.Globalization;

    /// <summary>
    /// Represents the ribbon for Creative Commons Microsoft Excel add-in.
    /// </summary>
    public partial class CreativeCommonsRibbonTab : OfficeRibbon
    {
        /// <summary>
        /// Image of the license.
        /// </summary>
        private Image license;

        /// <summary>
        /// Creates an instance of this class.
        /// </summary>
        public CreativeCommonsRibbonTab()
        {
            this.InitializeComponent();
            ThisAddIn.ApplicationStartedEvent += new EventHandler(this.ThisAddInApplicationStarted);
        }

        /// <summary>
        /// Sets the license image in ribbon UI.
        /// </summary>
        private Image LicenseImage
        {
            get
            {
                return this.license;
            }
            set
            {
                LicenseInfo licenseInfo = Helper.ReadLicenseInfo((DocumentProperties)ThisAddIn.ExcelApplication.ActiveWorkbook.CustomDocumentProperties);
                this.licensedGallery.Items[0].Label = Helper.GetLicenseLabel(licenseInfo.Id);
                this.licensedGallery.Items[0].Image = value;
                this.license = value;
            }
        }



        /// <summary>
        /// Sets a value indicating whether the workbook is licensed.
        /// </summary>
        private void WorkbookLicensed(bool value)
        {
            this.licensedGallery.Visible = value;
            this.notLicensedButton.Visible = !value;
        }

        /// <summary>
        /// Sets a value indicating whether the ribbon UI is enabled.
        /// </summary>
        private bool RibbonEnabled
        {
            get
            {
                return this.licensesRibbonGallery.Enabled;
            }
            set
            {
                this.licensedGallery.Enabled = value;
                this.notLicensedButton.Enabled = value;
                this.licensesRibbonGallery.Enabled = value;
            }
        }

        /// <summary>
        /// Prepares the cache folders.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void CreativeCommonsRibbonTabLoad(object sender, RibbonUIEventArgs e)
        {
            Helper.PrepareLicenseFolders();

            // Set localized labels
            this.tabAttributes.Label = Helper.GetString("CreativeCommons");
            this.groupCreativeCommons.Label = Helper.GetString("CreativeCommons");
            this.licensedGallery.Label = Helper.GetString("Licensed", new object[] { "Workbook" });
            this.notLicensedButton.Label = Helper.GetString("NotLicensed", new object[] { "Workbook" });
            this.licensesRibbonGallery.Label = Helper.GetString("Licenses");
            this.removeLicenseButton.Label = Helper.GetString("RemoveLicenseFromDocument", new object[] { "Workbook" });
            this.newLicenseButton.Label = Helper.GetString("NewLicense");
            this.removeLicensesButton.Label = Helper.GetString("RemoveLicensesFromDisk");
            this.notLicensedButton.SuperTip = Helper.GetString("LicenseIndicatorToolTip");
            this.licensedGallery.SuperTip = Helper.GetString("LicenseGalleryToolTip");
            this.licensesRibbonGallery.SuperTip = Helper.GetString("LicensesGalleryToolTip");

            this.newLicenseButton.Click += this.NewLicense;
            this.removeLicenseButton.Click += new EventHandler<RibbonControlEventArgs>(this.RemoveLicenseFromDocumentButtonClick);
            this.removeLicensesButton.Click += new EventHandler<RibbonControlEventArgs>(this.RemoveLicensesButtonClick);
            this.licensedGallery.ItemsLoading += new EventHandler<RibbonControlEventArgs>(this.LicensedGalleryItemsLoading);
        }

        /// <summary>
        /// Updates the ribbon UI.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void LicensedGalleryItemsLoading(object sender, RibbonControlEventArgs e)
        {
            this.UpdateRibbonUI();
        }

        /// <summary>
        /// Associates the event handlers when application starts.
        /// </summary>
        private void ThisAddInApplicationStarted(System.Object sender, System.EventArgs e)
        {
            ThisAddIn.ExcelApplication.WindowActivate += new AppEvents_WindowActivateEventHandler(this.ExcelApplicationWindowActivate);
            ThisAddIn.ExcelApplication.WorkbookOpen += new AppEvents_WorkbookOpenEventHandler(this.ExcelApplicationWorkbookOpen);

            // This call is required to handle the scenario when the add-in is disabled via Options and enabled again.
            this.UpdateRibbonUI();
        }

        /// <summary>
        /// Updates the ribbon UI when a workbook is opened.
        /// </summary>
        /// <param name="wb">Workbook object.</param>
        private void ExcelApplicationWorkbookOpen(Workbook wb)
        {
            this.UpdateRibbonUI();
        }

        /// <summary>
        /// Updates the ribbon UI when a document is closed.
        /// </summary>
        /// <param name="wb">Workbook object.</param>
        /// <param name="cancel">A value indicating if the workbook can be closed.</param>
        private void ExcelApplicationWorkbookBeforeClose(Workbook wb, ref bool cancel)
        {
            // When Excel is opened without a presentation.
            this.WorkbookLicensed(false);
            this.RibbonEnabled = false;
        }

        /// <summary>
        /// Updates the ribbon UI when user switches between workbooks.
        /// </summary>
        /// <param name="wb">Workbook object.</param>
        /// <param name="wn">Excel window.</param>
        private void ExcelApplicationWindowActivate(Workbook wb, Window wn)
        {
            this.UpdateRibbonUI();
        }

        /// <summary>
        /// Invokes the "Remove licenses from disk" dialog.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void RemoveLicensesButtonClick(object sender, RibbonControlEventArgs e)
        {
            RemoveLicensesDialog dlg = new RemoveLicensesDialog();
            dlg.ShowDialog();
        }

        /// <summary>
        /// Removes the Creative Commons license from the workbook.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void RemoveLicenseFromDocumentButtonClick(object sender, RibbonControlEventArgs e)
        {
            if (null != Helper.ReadLicenseInfo((DocumentProperties)ThisAddIn.ExcelApplication.ActiveWorkbook.CustomDocumentProperties))
            {
                if (!Helper.GetUserResponse(Helper.GetString("RemoveLicenseMessage", new object[] { "workbook" })))
                {
                    return;
                }
            }

            ThisAddIn.RemoveLicenseFromDocument();
            ThisAddIn.ExcelApplication.ActiveWorkbook.Saved = false;
            this.UpdateRibbonUI();
        }


        /// <summary>
        /// Loads and displays the available licenses in the ribbon UI.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void LoadLicenses(object sender, RibbonControlEventArgs e)
        {
            this.UpdateRibbonUI();
            this.licensesRibbonGallery.Items.Clear();

            if (Directory.Exists(Helper.LocalLicensesFolderPath))
            {
                foreach (string dir in Directory.GetDirectories(Helper.LocalLicensesFolderPath))
                {
                    RibbonDropDownItem item = new RibbonDropDownItem();
                    item.Tag = dir.Substring(Helper.LocalLicensesFolderPath.Length + @"\".Length);

                    if (FileAttributes.Hidden != (File.GetAttributes(dir) & FileAttributes.Hidden))
                    {
                        continue;
                    }

                    if (!File.Exists(Path.Combine(dir, Helper.LicenseFileName)) || !File.Exists(Path.Combine(dir, Helper.LicenseImageFileName)))
                    {
                        continue;
                    }

                    try
                    {
                        item.Image = Helper.GetImageFromFile(Path.Combine(dir, Helper.LicenseImageFileName));
                    }
                    catch (FileNotFoundException)
                    {
                        // Ignore if the icon file is missing.
                    }

                    this.licensesRibbonGallery.Items.Add(item);
                    this.licensesRibbonGallery.Items[this.licensesRibbonGallery.Items.Count - 1].Label = Helper.GetLicenseLabel(item.Tag.ToString());
                }
            }

            // Disable 'Remove Licenses' button if there is no license is available for removal.
            this.removeLicensesButton.Visible = !(0 == this.licensesRibbonGallery.Items.Count);
        }

        /// <summary>
        /// Invokes the "New license" wizard.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void NewLicense(object sender, RibbonControlEventArgs e)
        {
            NewLicenseWizard newLicenseWizard = new NewLicenseWizard();
            newLicenseWizard.ShowDialog();
        }

        /// <summary>
        /// Associates the selected Creative Commons license with the workbook.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event arguments.</param>
        private void AssociateLicense(object sender, RibbonControlEventArgs e)
        {
            if (null != Helper.ReadLicenseInfo((DocumentProperties)ThisAddIn.ExcelApplication.ActiveWorkbook.CustomDocumentProperties))
            {
                if (!Helper.GetUserResponse(Helper.GetString("ReAssociateConfirmMessage", new object[] { "workbook" })))
                {
                    return;
                }
            }

            ThisAddIn.RemoveLicenseFromDocument();


            Image img = this.licensesRibbonGallery.Items[this.licensesRibbonGallery.SelectedItemIndex].Image;

            string licname = this.licensesRibbonGallery.Items[this.licensesRibbonGallery.SelectedItemIndex].Tag.ToString();
            string[] licenseLogoInfo = Helper.GetLicenseInfo(licname);
            licenseLogoInfo[0] = RemoveJunkCharactersFromLicenseText(licenseLogoInfo[0]);
            string imgFilePath = Path.Combine(Path.Combine(Helper.LocalLicensesFolderPath, licname), Helper.LicenseImageFileName);

            //Inserting a row after coping multiple rows will result in pasting copied rows
            //To avoid this issue doing some operation to remove the cell copy 
            //This will not clear any object copy. 
            bool wraptext = (bool)ThisAddIn.ExcelApplication.ActiveCell.WrapText;
            ThisAddIn.ExcelApplication.ActiveCell.WrapText = !wraptext;

            // empty row before license
            ThisAddIn.ExcelApplication.ActiveCell.EntireRow.Insert(Type.Missing, XlInsertFormatOrigin.xlFormatFromLeftOrAbove);
            // rows to insert license image and text
            ThisAddIn.ExcelApplication.ActiveCell.EntireRow.Insert(Type.Missing, XlInsertFormatOrigin.xlFormatFromLeftOrAbove);
            ThisAddIn.ExcelApplication.ActiveCell.EntireRow.Insert(Type.Missing, XlInsertFormatOrigin.xlFormatFromLeftOrAbove);

            //Insert the license image to the workbook
            Worksheet sheet = (Worksheet)ThisAddIn.ExcelApplication.ActiveSheet;
            Range cellRange = (Range)sheet.Cells[ThisAddIn.ExcelApplication.ActiveCell.Row + 4, ThisAddIn.ExcelApplication.ActiveCell.Column + 1];
            cellRange = (Range)sheet.Cells[ThisAddIn.ExcelApplication.ActiveCell.Row + 1, ThisAddIn.ExcelApplication.ActiveCell.Column];
            if (null != img)
            {
                cellRange.RowHeight = img.Height * Helper.PixelPointRatio;
                cellRange.ColumnWidth = img.Width * Helper.PixelPointRatio;

                cellRange.Select();
                //insert the license image to sheet
                sheet.Shapes.AddPicture(imgFilePath,
                    Microsoft.Office.Core.MsoTriState.msoFalse,
                    Microsoft.Office.Core.MsoTriState.msoCTrue,
                    Convert.ToSingle(cellRange.Left, CultureInfo.CurrentCulture),
                    Convert.ToSingle(cellRange.Top, CultureInfo.CurrentCulture),
                    Convert.ToSingle((img.PhysicalDimension.Width * Helper.PixelPointRatio), CultureInfo.CurrentCulture),
                    Convert.ToSingle((img.PhysicalDimension.Height * Helper.PixelPointRatio), CultureInfo.CurrentCulture));
            }

            //Insert license hyperlink to sheet
            cellRange = (Range)sheet.Cells[ThisAddIn.ExcelApplication.ActiveCell.Row + 1, ThisAddIn.ExcelApplication.ActiveCell.Column];
            cellRange.Select();
            cellRange.Hyperlinks.Add(cellRange, licenseLogoInfo[1], Type.Missing, Type.Missing, licenseLogoInfo[2] + licenseLogoInfo[0] + licenseLogoInfo[3]);
            cellRange.EntireColumn.AutoFit();


            //Save the license info into document properties
            DocumentProperties customProperties = (DocumentProperties)ThisAddIn.ExcelApplication.ActiveWorkbook.CustomDocumentProperties;
            LicenseInfo licenseInfo = new LicenseInfo();
            licenseInfo.Id = this.licensesRibbonGallery.Items[this.licensesRibbonGallery.SelectedItemIndex].Tag.ToString();
            licenseInfo.LicenseUrl = licenseLogoInfo[2];
            licenseInfo.LicenseXml = licenseLogoInfo[4];
            Helper.SaveLicenseInfo((DocumentProperties)ThisAddIn.ExcelApplication.ActiveWorkbook.CustomDocumentProperties, licenseInfo);
            ThisAddIn.ExcelApplication.ActiveWorkbook.Saved = false;
            //Update the image on license
            this.license = img;

            // Insert an empty row after license
            cellRange = (Range)sheet.Cells[ThisAddIn.ExcelApplication.ActiveCell.Row + 1, ThisAddIn.ExcelApplication.ActiveCell.Column];
            cellRange.Select();
            cellRange.Insert(Type.Missing, XlInsertFormatOrigin.xlFormatFromLeftOrAbove);
            //Revert the wrap text
            ThisAddIn.ExcelApplication.ActiveCell.WrapText = wraptext;
            //Update the ribbon 
            this.UpdateRibbonUI();
        }

        /// <summary>
        /// Updates the ribbon UI based on the state of Excel and presentation.
        /// </summary>
        private void UpdateRibbonUI()
        {
            if (null != ThisAddIn.ExcelApplication)
            {
                // When PowerPoint is opened without a workbook.
                if (!ThisAddIn.HasWorkbook())
                {
                    this.WorkbookLicensed(false);
                    this.RibbonEnabled = false;
                }

                // When the workbook does not have a license.
                else if (null == Helper.ReadLicenseInfo((DocumentProperties)ThisAddIn.ExcelApplication.ActiveWorkbook.CustomDocumentProperties))
                {
                    this.WorkbookLicensed(false);
                    this.RibbonEnabled = true;
                }

                // When the workbook has a license.
                else
                {
                    LicenseInfo licenseInfo = Helper.ReadLicenseInfo((DocumentProperties)ThisAddIn.ExcelApplication.ActiveWorkbook.CustomDocumentProperties);
                    this.RibbonEnabled = true;
                    this.WorkbookLicensed(true);
                    this.LicenseImage = licenseInfo.Image;
                }
            }
        }

        /// <summary>
        /// Removes redundant characters like \r & \n from the string, also removes some extra spaces
        /// </summary>
        /// <param name="str">string from which to remove the junk characters</param>
        /// <returns>Modified string</returns>
        private string RemoveJunkCharactersFromLicenseText(string str)
        {
            try
            {
                StringBuilder strBuilder = new StringBuilder(str);
                strBuilder = strBuilder.Replace("\r\n ", "");
                strBuilder = strBuilder.Replace("\r\n", "");
                str = strBuilder.ToString();
            }
            catch (Exception)
            {
            }
            return str;
        }
    }
}
